
using Plots
using Statistics      
using DelimitedFiles  
using LinearAlgebra  
using CSV, DataFrames
using LaTeXStrings
using PyCall
using Conda
using LsqFit

function loadFile(path)
    headerdata = readdlm(path)
    #headerdata = CSV.read(path)
    
    #Esto selecciona sólo los renglones de la matríz que tienen datos del voltaje:
    EEG_data = convert(Array{Float64},
                       headerdata[18:end,:]
                      )
    timescale = collect(1:size(EEG_data)[1])*1/512
    return(EEG_data)
end

function loadFile_TDT(path)
    #headerdata = readdlm(path)
    headerdata = CSV.read(path)
    
    #Esto selecciona sólo los renglones de la matríz que tienen datos del voltaje:
    EEGdat = convert(Array{Float64},headerdata[3:end,:])
                      
   
    timescale = collect(1:size(EEGdat)[1])*1/512
    return(EEGdat)
end

"Returns an array which elements are slided time windows taken from 'data'.
These time windows have lenght of 'npoints'. The window slide is of lenght 'stepsize'.
    
ex.
a = [1,2,3,4,5,6,7]
b = windowed_data(a,2,1)
b
[1, 2, 3]
[2, 3, 4]
[3, 4, 5]
[4, 5, 6]
[5, 6, 7]"

function windowed_data(data,npoints,stepsize)

    datapoints = size(data)[1] #Número de puntos de la serie de tiempo
    anchorpoints = collect(1:stepsize:datapoints-npoints)  #Índices ancla (en los que empieza la ventana de tiempo)
    n_overlap_wind = fld(datapoints,npoints)  # Número de ventanas traslapables
    winlen = fld(datapoints,npoints)
    
    ov_data = []
    
    for i in 1:length(anchorpoints)
        ov_w = data[anchorpoints[i]:anchorpoints[i]+npoints,:]
        
        push!(ov_data,ov_w)
    end
    
    return(ov_data)
    #writedlm("/Users/nelion/Desktop/Tesis_Resultados_Marzo2019/$npoints$anchor$name",EVflux)
end

 #Regresa la varianza explicada por las primeras M CP's en cada ventana de tiempo

function Explained_Variance(data, npoints, M)
    
    covmat = cov(data)
    #Toma los pesos eigenvalores asociados a los eigenvectores de las matríces de covarianza:
    Evals = eigvals(covmat)
    PCAscores = Evals/sum(Evals)
    PCA_dimred_scores = [sum(reverse(PCAscores[end-M:end]))][1]

    return(PCA_dimred_scores)
end

function PCA_projection(data)
    
    covmat = cov(data)
    p_components = eigvecs(covmat)
    reduced_pc = p_components[:,end-2:end] 
   
    PCA_projection = transpose(reduced_pc) * transpose(data)
    
    return(transpose(PCA_projection))

end

function EVFlux_overlap(data,npoints,anchor,ncomps)
    #npoints es el tamaño del "paso" de la ventana de tiempo
    
    datapoints = size(data)[1]
    nsignals = size(data)[2]
    anchorpoints = collect(1:anchor:datapoints-npoints)  #Puntos ancla (en los que empieza la ventana de tiempo)
    n_overlap_wind = fld(datapoints,npoints)  # Número de ventanas traslapables
    winlen = fld(datapoints,npoints)
    #name = "tom"
    
    ov_data = []
    
    for i in 1:length(anchorpoints)
        ov_w = data[anchorpoints[i]:npoints+anchorpoints[i],:]
        
        push!(ov_data,ov_w)
    end
    
    EVflux = [Explained_Variance(window,npoints,ncomps)[1] for window in ov_data]
    return(EVflux)
    #writedlm("/Users/nelion/Desktop/Tesis_Resultados_Marzo2019/$npoints$anchor$name",EVflux)
end

Conda.add("scipy")
fft = pyimport("scipy.fftpack")


function lowHighCutFreqSmoothing(x,lowCut=0,hiCut=500)
    
    rft = fft.rfft(x); #transformada de Fourier de la serie (FREQ)
    rft[1:lowCut] = zero(1:lowCut)         #Filtro pasa altas
    rft[hiCut:end] = zero(hiCut:length(x)) #Filtro pasa bajas (corta frecuencias más altas que highcut)
    #y_smooth = sc.ifft(rft, N)
    y = fft.irfft(rft) #Regreso al dominio del tiempo
    return(y) #,y_smooth
end


function diff_TS(data)
"""
    Tasa de cambio de un array (diferencia entre valores consecutivos)
"""    
    diff_TS = [data[i+1]-data[i] for i in 1:length(data)-1]
    return(diff_TS)
end

@. exp_fit(x, p) = p[1]*exp(-x*p[2])

@. lin_model(x, p) = (p[1]*x)+p[2]

@. quad_model(x, p) = (p[1]*x^2)+p[2]

#xdata = collect(1:size(oscil_data)[1])
#ydata = oscil_data[:,1]

#p0 = [0.5, 0.5]
#fit = curve_fit(quad_model, xdata, ydata, p0)

#fit_ydata = quad_model(xdata,[coef(fit)[1],coef(fit)[2]])
